import netgen.gui
import ngsolve
from ngsolve import Draw, Redraw # just for visualization 
from netgen.csg import *
from ngsolve.internal import visoptions
from ngsolve.internal import viewoptions
from ngsolve.solvers import *
import math
from math import pi, acos, degrees
from netgen.meshing import Element0D, Element1D, Element2D, Element3D, MeshPoint, FaceDescriptor, Mesh
import scipy.sparse as sp
import time
import itertools

geo = CSGeometry()
l = 1
bot   = Plane (Pnt(0,0,0), Vec(0,0,-1) )
s0   = Plane (Pnt(0,l,0), Vec(0.866,0.5,0.353) )
s1   = Plane (Pnt(-0.866*l,-0.5*l,0), Vec(-0.866,0.5,0.353) )
s2   = Plane (Pnt(0.866*l,-0.5*l,0), Vec(0,-1,0.353) )
tetrahedron = bot*s0*s1*s2
geo.Add (tetrahedron.mat("tetrahedron"))

ngmesh = geo.GenerateMesh(maxh=2.0)
mesh = ngsolve.Mesh(ngmesh)

vertices = []
for v in mesh.vertices:
    vertices.append(v.nr+1)

print("")
print("Mesh vertices:")
print(vertices)

emap = {}
elements = []

print("")
print("Mesh elements:")

for e in mesh.ngmesh.Elements3D():
    elements.append(e)
    
for i in range(0, mesh.ne):
    e = NodeId(ELEMENT,i)
    vlist = []
    for v in mesh[e].vertices:
        vlist.append(v.nr+1)
    print(vlist)

mesh.ngmesh.Refine()
Draw(mesh)

vertices = []
for v in mesh.vertices:
    vertices.append(v.nr+1)

print("")    
print("Refined mesh vertices:")
print(vertices)

emap = {}
elements = []

print("")
print("Refined mesh elements:")

for e in mesh.ngmesh.Elements3D():
    elements.append(e)
    
for i in range(0, mesh.ne):
    e = NodeId(ELEMENT,i)
    vlist = []
    for v in mesh[e].vertices:
        vlist.append(v.nr+1)
    print(vlist)
    l = list(itertools.permutations([vlist[0],vlist[1],vlist[2],vlist[3]]))
    for p in l:
        emap[p] = elements[e.nr]
        
v1  = vertices[0]
v2  = vertices[1]
v3  = vertices[2]
v4  = vertices[3]
v5  = vertices[4]
v6  = vertices[5]
v7  = vertices[6]
v8  = vertices[7]
v9  = vertices[8]
v10 = vertices[9]

print("")
print("Exploring the refined mesh:")

if (v1, v5, v6, v7) in emap:                       
    print("(v1, v5, v6, v7 ) ->",emap[(v1, v5, v6, v7 )])
if (v2, v5, v8, v9) in emap:                       
    print("(v2, v5, v8, v9 ) ->",emap[(v2, v5, v8, v9 )])
if (v1, v5, v6, v7) in emap:                       
    print("(v3, v6, v8, v10) ->",emap[(v3, v6, v8, v10)])
if (v2, v5, v8, v9) in emap:                       
    print("(v4, v7, v9, v10) ->",emap[(v4, v7, v9, v10)])
                                                
v6_v9_diagonal = False
if (v6, v9, v5, v7) in emap:
    if (v6, v9, v7, v10) in emap:
        if (v6, v9, v10, v8) in emap:
            if (v6, v9, v8, v5) in emap:
                v6_v9_diagonal = True
                print("")
                print("Internal octahedron has diagonal v6-v9!")
                print("(v6, v9, v5,  v7 ) ->",emap[(v6, v9, v5,  v7 )])
                print("(v6, v9, v7,  v10) ->",emap[(v6, v9, v7,  v10)])
                print("(v6, v9, v10, v8 ) ->",emap[(v6, v9, v10, v8 )])
                print("(v6, v9, v8,  v5 ) ->",emap[(v6, v9, v8,  v5 )])
                
if v6_v9_diagonal:
    print("")
    print("I want to change to diagonal v5-v10:")
    
    #emap[(v6, v9, v5,  v7 )].vertices = [v5, v10, v6, v7]
    #emap[(v6, v9, v7,  v10)].vertices = [v5, v10, v7, v9]
    #emap[(v6, v9, v10, v8 )].vertices = [v5, v10, v9, v8]
    #emap[(v6, v9, v8,  v5 )].vertices = [v5, v10, v8, v6]
    
    emap[(v6, v9, v5,  v7 )].points = [v5, v10, v6, v7]
    emap[(v6, v9, v7,  v10)].points = [v5, v10, v7, v9]
    emap[(v6, v9, v10, v8 )].points = [v5, v10, v9, v8]
    emap[(v6, v9, v8,  v5 )].points = [v5, v10, v8, v6]


